@@ -2,7 +2,7 @@ import os
 import uuid
 import logging
 from functools import lru_cache
-from copy import copy
+from copy import copy, deepcopy
 from collections import defaultdict
 from typing import Mapping, Tuple
 from linkml_runtime.utils.namespaces import Namespaces
@@ -136,7 +136,7 @@ class SchemaView(object):
         return schema
 
     @lru_cache()
-    def imports_closure(self, traverse=True) -> List[SchemaDefinitionName]:
+    def imports_closure(self, traverse=True, inject_metadata=True) -> List[SchemaDefinitionName]:
         """
         Return all imports
 
@@ -162,8 +162,16 @@ class SchemaView(object):
             for i in s.imports:
                 if i not in visited:
                     todo.append(i)
+        if inject_metadata:
+            for s in self.schema_map.values():
+                for x in {**s.classes, **s.enums, **s.slots, **s.subsets}.values():
+                    x.from_schema = s.id
+                for c in s.classes.values():
+                    for a in c.attributes.values():
+                        a.from_schema = s.id
         return closure
 
+
     @lru_cache()
     def all_schema(self, imports: True) -> List[SchemaDefinition]:
         """
@@ -192,20 +200,26 @@ class SchemaView(object):
 
     @deprecated("Use `all_slots` instead")
     @lru_cache()
-    def all_slot(self, imports=True) -> Dict[SlotDefinitionName, SlotDefinition]:
+    def all_slot(self, **kwargs) -> Dict[SlotDefinitionName, SlotDefinition]:
         """
         :param imports: include imports closure
         :return: all slots in schema view
         """
-        return self._get_dict(SLOTS, imports)
+        return self.all_slots(**kwargs)
 
     @lru_cache()
-    def all_slots(self, imports=True) -> Dict[SlotDefinitionName, SlotDefinition]:
+    def all_slots(self, imports=True, attributes=True) -> Dict[SlotDefinitionName, SlotDefinition]:
         """
         :param imports: include imports closure
         :return: all slots in schema view
         """
-        return self._get_dict(SLOTS, imports)
+        d = copy(self._get_dict(SLOTS, imports))
+        if attributes:
+            for c in self.all_classes().values():
+                for aname, a in c.attributes.items():
+                    d[aname] = a
+        return d
+
 
     @deprecated("Use `all_enums` instead")
     @lru_cache()
@@ -290,12 +304,29 @@ class SchemaView(object):
     def _get_dict(self, slot_name: str, imports=True) -> Dict:
         schemas = self.all_schema(imports)
         d = {}
+        # iterate through all schemas and merge the list together
         for s in schemas:
             # get the value of element name from the schema, if empty, return empty dictionary.
             d1 = getattr(s, slot_name, {})
             d = {**d, **d1}
+
         return d
 
+    @lru_cache()
+    def slot_name_mappings(self) -> Dict[str, SlotDefinition]:
+        """
+        Mapping between processed safe slot names and slots.
+
+        For example, a slot may have name 'lives at', the code-safe version is `lives_at`
+        :return: mapping from safe names to slot
+        """
+        m = {}
+        for s in self.all_slots().values():
+            m[underscore(s.name)] = s
+
+        return m
+
+
     @lru_cache()
     def in_schema(self, element_name: ElementName) -> SchemaDefinitionName:
         """
@@ -303,6 +334,8 @@ class SchemaView(object):
         :return: name of schema in which element is defined
         """
         ix = self.element_by_schema_map()
+        if element_name not in ix:
+            raise Exception(f'Element {element_name} not in any schema')
         return ix[element_name]
 
     @lru_cache()
@@ -313,6 +346,9 @@ class SchemaView(object):
             for type_key in [CLASSES, SLOTS, TYPES, ENUMS, SUBSETS]:
                 for k, v in getattr(schema, type_key, {}).items():
                     ix[k] = schema.name
+            for c in schema.classes.values():
+                for aname, a in c.attributes.items():
+                    ix[aname] = schema.name
         return ix
 
     @lru_cache()
@@ -343,6 +379,7 @@ class SchemaView(object):
                         # slot name is ambiguous, no results
                         return None
                     slot = copy(c.attributes[slot_name])
+                    slot.from_schema = c.from_schema
                     slot.owner = c.name
         if strict and slot in None:
             raise Exception(f'No such slot as "{slot_name}"')
@@ -484,6 +521,19 @@ class SchemaView(object):
         """
         return _closure(lambda x: self.class_children(x, imports=imports, mixins=mixins, is_a=is_a), class_name, reflexive=reflexive)
 
+    @lru_cache()
+    def slot_descendants(self, slot_name: SLOT_NAME, imports=True, mixins=True, reflexive=True, is_a=True) -> List[SlotDefinitionName]:
+        """
+        Closure of slot_children method
+
+        :param slot_name: query slot
+        :param imports: include import closure
+        :param mixins: include mixins (default is True)
+        :param is_a: include is_a parents (default is True)
+        :param reflexive: include self in set of descendants
+        :return: descendants slot names
+        """
+        return _closure(lambda x: self.slot_children(x, imports=imports, mixins=mixins, is_a=is_a), slot_name, reflexive=reflexive)
 
     @lru_cache()
     def class_roots(self, imports=True, mixins=True, is_a=True) -> List[ClassDefinitionName]:
@@ -584,8 +634,13 @@ class SchemaView(object):
         else:
             raise Exception(f'Must be class or slot or type: {e}')
         if uri is None or native:
-            schema = self.schema_map[self.in_schema(e.name)]
-            ns = self.namespaces()
+            if e.from_schema is not None:
+                schema = next(sc for sc in self.schema_map.values() if sc.id == e.from_schema)
+                if schema == None:
+                    raise Exception(f'Cannot find {e.from_schema} in schema_map')
+            else:
+                logging.warning(f'from_schema not populated for element {e.name}')
+                schema = self.schema_map[self.in_schema(e.name)]
             pfx = schema.default_prefix
             uri = f'{pfx}:{e_name}'
         if expand:
@@ -731,7 +786,7 @@ class SchemaView(object):
         cls = self.get_class(class_name, imports)
         islot = None
         if slot is not None:
-            islot = copy(slot)
+            islot = deepcopy(slot)
         else:
             for an in self.class_ancestors(class_name):
                 a = self.get_class(an, imports)
@@ -769,7 +824,9 @@ class SchemaView(object):
                     v = self.schema.default_range
             if v is not None:
                 setattr(islot, metaslot_name, v)
-        return islot
+        if slot.inlined_as_list:
+            slot.inlined = True
+        return deepcopy(islot)
 
     @lru_cache()
     def class_induced_slots(self, class_name: CLASS_NAME = None, imports=True) -> List[SlotDefinition]:
@@ -805,14 +862,19 @@ class SchemaView(object):
         """
         if slot.inlined:
             return True
+        elif slot.inlined_as_list:
+            return True
         else:
             range = slot.range
-            id_slot = self.get_identifier_slot(range, imports=imports)
-            if id_slot is None:
-                # must be inlined as has no identifier
-                return True
+            if range in self.all_classes():
+                id_slot = self.get_identifier_slot(range, imports=imports)
+                if id_slot is None:
+                    # must be inlined as has no identifier
+                    return True
+                else:
+                    # not explicitly declared inline and has an identifier: assume is ref, not inlined
+                    return False
             else:
-                # not explicitly declared inline and has an identifier: assume is ref, not inlined
                 return False
 
     @lru_cache()
